unit UConduct;

//Source code based on old Fortran file

interface
  uses SysUtils, Dialogs, UDMEnDrain, USpacing;
  var QL, QH, QQ : real;
  procedure CheckConductData;
  procedure DefineData;
  procedure DarCond;
  procedure EneCond;

implementation

uses UMainForm;

procedure CheckConductData;
{-------------------------}
var InputFile, ErrorFile : textfile;
begin
  assignfile(InputFile,DMEnDrain.FileName);
  reset (InputFile);
  readln (InputFile);
  readln (InputFile);
  readln (InputFile);
  readln (InputFile);
  readln (InputFile,RR,D2,DW,DB,RE,WD,DM,XX);
  closefile (InputFile);
//Data checks
  assignfile(ErrorFile,'Error.lst');
  rewrite (ErrorFile);
  writeln (ErrorFile,' THE FOLLOWING ERRORS WERE FOUND IN THE INPUT:');
  writeln (ErrorFile);
  InputError := false;
  if (RR<=0) then
  begin
    writeln (ErrorFile,' Make sure that R > 0.');
    InputError := true;
  end;
  D1:=D2;
  D1W := D1 - DW;
  if (D1W<0) then
  begin
    writeln (ErrorFile,' Make sure that D1 > Dw.');
    InputError := true;
  end;
  D21 := D2 - D1;
  if (D21<0) then
  begin
    writeln (ErrorFile,' Make sure that D2 > D1.');
    InputError := true;
  end;
  DBW := DB - DW;
  if (DBW<=0) then
  begin
    writeln (ErrorFile,' Make sure that Db > Dw.');
    InputError := true;
  end;
  DB1 := DB - D1;
  if (RE<0) then
  begin
    writeln (ErrorFile,' Make sure that E >= 0.');
    InputError := true;
  end;
  WX := WD/2;
(*
  if (D21<>0) and (D1<DB+WX) and (D1>DB) then
  begin
    writeln (ErrorFile,' Make sure that D1 > Db+Wd/2 or D1 < Db.');
    InputError := true;
  end;
  if (D2>DB) and (D2<DB+WX) then
  begin
    writeln (ErrorFile,' Make sure that D2 < Db or D2 > Db+Wd/2.');
    InputError := true;
  end;
*)
  ZM := DW - DM;
  if (ZM<=0) then
  begin
    writeln (ErrorFile,' Make sure that Dm < Dw.');
    InputError := true;
  end;
  if InputError then
  begin
    writeln (ErrorFile);
    writeln (ErrorFile, 'Please correct the errors.');
  end;
  closefile (ErrorFile);
end; {procedure CheckConductData}


Procedure DefineData;
{-------------------}
begin
  DR1:=D1W;
  DR2:=D21;
  DRT:=DR1+DR2;
  BWR1:=DBW;
//  B1R2:=DB1;
  B1R1:=DB1;
  IF (D1>DB) THEN
  begin
    DF:=WX-BWR1;
    IF (DF<0) then DF:=0;
    DG:=BWR1-WX;
    IF (DG<0) then DG:=0;
  end ELSE
  begin
    IF (D2>DB) THEN
    begin
//      DF:=WX-B1R2;
      DF:=WX-B1R1;
      IF (DF<0) then DF:=0;
//      DG:=B1R2-WX;
      DG:=B1R1-WX;
      IF (DG<0) then DG:=0;
    END else
    begin
      WX:=0;
      DF:=0;
//      DG:=B1R2;
      DG:=B1R1;
      IF (DG<0) then DG:=0
    END;
  END;
  PP:=0.0001;
  XX:=XX/2;
  NU:=round(XX/PP);
  KK:=1+round(WX/PP);
  IF (KK<2) then KK:=2;
  QL:=0.001;
  QH:=1000;
  QQ:=RR;
  YR:=100;
  YH:=1000;
  YL:=0;
  YRP:=0;
  RES:=0.0001;
  FE:=2*RR*XX*RE;
end; {procedure DefineData}


procedure DarCond;
{----------------}
label 1, 2, 3, 4, 5;
var IJ, II : integer;
begin
  InFileName:=DMEnDrain.FileName;
  OutFileName:=changefileext (DMEnDrain.FileName,'.out');
  assignfile(OutputFile,OutFileName);
  rewrite (OutputFile);
  writeln (OutputFile);
  writeln (OutputFile,' EnDrain, by R.J.Oosterbaan');
  writeln (OutputFile);
  writeln (OutputFile,' Program DarCond calculates the equivalent hydraulic'+
                      ' conductivity.');
  writeln (OutputFile,' of the soil between drains using the Darcy equation.');
  writeln (OutputFile,' The shape of the water table is calculated both'+
                      ' with the');
  writeln (OutputFile,' Darcy and energy balance equation.');
  writeln (OutputFile,' Calculations can be made for pipe drains or ditches');
  writeln (OutputFile,' with or without entrance resistance.');
  writeln (OutputFile);
  writeln (OutputFile,'  ',DMEnDrain.Title1);
  writeln (OutputFile,'  ',DMEnDrain.Title2);
  writeln (OutputFile);
  writeln (OutputFile,' Name of input file used:  ',InFileName);
  writeln (OutputFile,' Name of this output file: ',OutFileName);
  writeln (OutputFile);
  writeln (OutputFile,' DarCond input data: ');
  writeln (OutputFile);
  writeln (OutputFile,' Time average recharge or discharge'+
                      '         R    (m/day) :  ',RR:10:4);
  writeln (OutputFile,' Bottom depth of 1st layer below s.s.'+
                      '       D1       (m) :  ',D1:10:2);
  writeln (OutputFile,' Bottom depth of 2nd layer below s.s.'+
                      '       D2       (m) :  ',D2:10:2);
  writeln (OutputFile,' Depth waterlevel in drain below s.s.'+
                      '       Dw       (m) :  ',DW:10:2);
  writeln (OutputFile,' Depth of drain bottom below s.s.'+
                      '           Db       (m) :  ',DB:10:2);
  writeln (OutputFile,' Entrance resistance at the drain'+
                      '           E    (day/m) :  ',RE:10:3);
  writeln (OutputFile,' Max. width of water body in the drain'+
                      '      W        (m) :  ',WD:10:3);
  writeln (OutputFile,' Depth watertable midway between drains'+
                      '     Dm       (m) :  ',DM:10:2);
  writeln (OutputFile,' Spacing betweeen the drains'+
                      '                S        (m) :  ',XX:10:2);
  writeln (OutputFile);
  writeln (OutputFile,' Watertable heights (see also the Symbols button):');
  writeln (OutputFile,'   X       F*        G*/p      F         G/p'+
                      '       T1        T2');
  writeln (OutputFile,' Output data:');

  DefineData;

//Calculating hydr. cond. Darcy
1:NU:=round(XX/PP);
  for IJ:=KK to NU do
  begin
    XS:=PP*(IJ-0.5);                                  //XS =distance from drain
    YY:=1.571*XS;
    YZ:=YY+DG;
    FE:=2*RR*XX*RE;
      QA:=QQ;
      QV1:=QQ;
      QQ1:=QQ;
      QQ2:=QQ;
      QB2:=QQ;
      QB1:=QQ;
      QV2:=QQ;
    IF (IJ=KK) THEN
    BEGIN
      YI:=FE;
      YA:=FE;
    END ELSE
      YA:=YI+0.5*ZI*PP;                                  //YA is WT level at XS
    IF (D1>DB) THEN
    begin
      TR:=QA*YA+QV1*DF;                                  //TRA = transmissivity
      TRA:=TR+QQ1*YZ;
      IF (YZ>DR1) then TRA:=TR+QQ1*DR1+QQ2*(YZ-DR1);
      IF (YZ>DRT) then TRA:=TR+QQ1*DR1+QQ2*DR2;
    end ELSE
      IF (D2<=DB) THEN
         TRA:=QA*YA+QB1*D1W+QB2*D21
      ELSE
      begin
         TR:=QA*YA+QB1*D1W+QV2*DF;
         TRA:=TR+QQ2*YZ;
         IF (YZ>DR2) then TRA:=TR+QQ2*DR2;
      END;
    XC:=XX-XS;
    ZI:=RR*XC/TRA;
    YI:=YI+ZI*PP;
  end; {for IJ:=KK to NU do}

  DIF:=ABS(YI-ZM);
  IF (DIF<RES) then GOTO 2;
  IF (YI<ZM) THEN
  begin
    QH:=QQ;
    QQ:=0.5*(QQ+QL);
  end ELSE
  begin
   QL:=QQ;
   QQ:=0.5*(QQ+QH);
  END;
  GOTO 1;

//Calculating WT depth Energy (YR or YZ)
2:YR:=500;
  YH:=1000;
  YL:=0;
  YRP:=0;
  FE:=2*RR*XX*RE;
3:for II:=KK to NU do
  begin
    XS:=PP*(II-0.5);                                  //XS =distance from drain
    YY:=1.571*XS;
    YZ:=YY+DG;
    IF (II=KK) THEN
    BEGIN
      YI:=FE;
      YA:=FE;
    END ELSE
      YA:=YI+0.5*ZI*PP;                                  //YA is WT level at XS
    IF (D1>DB) THEN
    begin
      TR:=QA*YA+QV1*DF;
      TRA:=TR+QQ1*YZ;
      IF (YZ>DR1) then TRA:=TR+QQ1*DR1+QQ2*(YZ-DR1);
      IF (YZ>DRT) then TRA:=TR+QQ1*DR1+QQ2*DR2;
    end ELSE
    begin
      IF (D2<=DB) THEN
         TRA:=QA*YA+QB1*D1W+QB2*D21
      ELSE
      begin
         TR:=QA*YA+QB1*D1W+QV2*DF;
         TRA:=TR+QQ2*YZ;
         IF (YZ>DR2) then TRA:=TR+QQ2*DR2;
      END;
    end;
//            IF (TRA.LE.0.) STOP 'Program stopped: transmissivity too sma
//     *ll, try a larger entrance head.'
    XC:=XX-XS;
    T1:=RR*XC/TRA;
    AY:=YA-YR;
    T2:=AY/XC;
    ZI:=T1+T2;
    YI:=YI+ZI*PP;
    IF (YI>YR) OR (ZI<0) then GOTO 4;
  end; { For II:=KK to NU do}

4:DIF:=ABS(YRP-YR);
  IF (DIF<RES) then GOTO 5;
  YRP:=YR;                      //final YI=YR is midway WT level energy balance
  IF (YI>YR) THEN
  begin
    YL:=YR;
    YR:=(YR+YH)/2;
  end ELSE
  begin
    YH:=YR;
    YR:=(YR+YL)/2;
  END;
  GOTO 3;

5:WriteOutput;

  RR:=RR*XX*XX;
  writeln (OutputFile,' #');
  XL:=2*XL;
  WTd := DW-YN;
  WTe := DW-YJ;
  writeln (OutputFile,' The average hydraulic conductivity (Darcy, m/d)'+
                      ' is: ',QQ:8:3);
  writeln (OutputFile,' #');
  writeln (OutputFile,' Midway hydr. head (Darcy,  m) is: ',YN:8:3);
  writeln (OutputFile,' Midway hydr. head (energy, m) is: ',YJ:8:3);
  writeln (OutputFile,' Midway W.T. depth (Darcy,  m) is: ',WTd:8:3);
  writeln (OutputFile,' Midway W.T. depth (energy, m) is: ',WTe:8:3);
  writeln (OutputFile,' #');
  closefile (OutputFile);

end; {procedure DarCond}


procedure EneCond;
{----------------}
label 1, 2, 3, 4, 5;
var IJ : integer;
    Passed0, Passed1, Passed2 : boolean;

begin
  InFileName:=DMEnDrain.FileName;
  OutFileName:=changefileext (DMEnDrain.FileName,'.out');
  assignfile(OutputFile,OutFileName);
  rewrite (OutputFile);
  writeln (OutputFile);
  writeln (OutputFile,' EnDrain, by R.J.Oosterbaan');
  writeln (OutputFile);
  writeln (OutputFile,' Program EneCond calculates the equivalent hydraulic'+
                      ' conductivity.');
  writeln (OutputFile,' of the soil between drains using the energy balance'+
                      ' equation.');
  writeln (OutputFile,' The shape of the water table is calculated both'+
                      ' with the');
  writeln (OutputFile,' Darcy and energy balance equation.');
  writeln (OutputFile,' Calculations can be made for pipe drains or ditches');
  writeln (OutputFile,' with or without entrance resistance.');
  writeln (OutputFile);
  writeln (OutputFile,'  ',DMEnDrain.Title1);
  writeln (OutputFile,'  ',DMEnDrain.Title2);
  writeln (OutputFile);
  writeln (OutputFile,' Name of input file used:  ',InFileName);
  writeln (OutputFile,' Name of this output file: ',OutFileName);
  writeln (OutputFile);
  writeln (OutputFile,' EneCond input data: ');
  writeln (OutputFile);
  writeln (OutputFile,' Time average recharge or discharge'+
                      '         R    (m/day) :  ',RR:10:4);
  writeln (OutputFile,' Bottom depth of 1st layer below s.s.'+
                      '       D1       (m) :  ',D1:10:2);
  writeln (OutputFile,' Bottom depth of 2nd layer below s.s.'+
                      '       D2       (m) :  ',D2:10:2);
  writeln (OutputFile,' Depth waterlevel in drain below s.s.'+
                      '       Dw       (m) :  ',DW:10:2);
  writeln (OutputFile,' Depth of drain bottom below s.s.'+
                      '           Db       (m) :  ',DB:10:2);
  writeln (OutputFile,' Entrance resistance at the drain'+
                      '           E    (day/m) :  ',RE:10:3);
  writeln (OutputFile,' Max. width of water body in the drain'+
                      '      W        (m) :  ',WD:10:3);
  writeln (OutputFile,' Depth watertable midway between drains'+
                      '     Dm       (m) :  ',DM:10:2);
  writeln (OutputFile,' Spacing betweeen the drains'+
                      '                S        (m) :  ',XX:10:2);
  writeln (OutputFile);
  writeln (OutputFile,' Watertable heights (see also the Symbols button):');
  writeln (OutputFile,'   X       F*        G*/p      F         G/p'+
                      '       T1        T2');
  writeln (OutputFile,' Output data:');

  DefineData;
  Passed0:=false;
  Passed1:=false;
  Passed2:=false;
//Calculating hydr. cond. energy balance

1:YR:=0.001;
  YH:=100;
  YL:=0;
  YRP:=0;

2:NU:=round(XX/PP);
  for IJ:=KK to NU do
  begin
    XS:=PP*(IJ-0.5);                                  //XS =distance from drain
    YY:=1.571*XS;
    YZ:=YY+DG;
    FE:=2*RR*XX*RE;
      QA:=QQ;
      QV1:=QQ;
      QQ1:=QQ;
      QQ2:=QQ;
      QB2:=QQ;
      QB1:=QQ;
      QV2:=QQ;
    IF (IJ=KK) THEN
    BEGIN
      YI:=FE;
      YA:=FE;
    END ELSE
      YA:=YI+0.5*ZI*PP;                                  //YA is WT level at XS
    IF (D1>DB) THEN
    begin
      TR:=QA*YA+QV1*DF;                                  //TRA = transmissivity
      TRA:=TR+QQ1*YZ;
      IF (YZ>DR1) then TRA:=TR+QQ1*DR1+QQ2*(YZ-DR1);
      IF (YZ>DRT) then TRA:=TR+QQ1*DR1+QQ2*DR2;
    end ELSE
      IF (D2<=DB) THEN
         TRA:=QA*YA+QB1*D1W+QB2*D21
      ELSE
      begin
         TR:=QA*YA+QB1*D1W+QV2*DF;
         TRA:=TR+QQ2*YZ;
         IF (YZ>DR2) then TRA:=TR+QQ2*DR2;
      END;
    XC:=XX-XS;
    T1:=RR*XC/TRA;
    AY:=YA-YR;
    T2:=AY/XC;
    ZI:=T1+T2;
    YI:=YI+ZI*PP;
    IF (YI>YR) OR (ZI<0) then GOTO 3;
  end; { For IJ:=KK to NU do}


3:DIF:=ABS(YRP-YR);
  IF (DIF<RES) then GOTO 4;
  YRP:=YR;                      //final YI=YR is midway WT level energy balance
  IF (YI>YR) THEN
  begin
    YL:=YR;
    YR:=(YR+YH)/2;
  end ELSE
  begin
    YH:=YR;
    YR:=(YR+YL)/2;
  END;
  GOTO 2;

4:DIF:=ABS(YI-ZM);
  IF (DIF<RES) then GOTO 5;
  if not Passed0 and (XX<=25) and (Dif<1000*RES) then
  begin
    MainForm.WaitMemo.Lines.Add(' Still working ...');
    Passed0:=true;
  end;
  if not Passed1 then
     if ((DIF<100*RES) and (XX>25) and (XX<50)) or
        ((DIF<1000*RES) and (XX>=50)) then
        begin
          MainForm.WaitMemo.Lines.Add(' Still working ...');
          Passed1:=true;
        end;
  if not Passed2 and (DIF<10*RES) and (XX>=50) then
  begin
    MainForm.WaitMemo.Lines.Add(' Almost done .....');
    Passed2:=true;
  end;
  IF (YR<ZM) THEN
  begin
    QH:=QQ;
    QQ:=0.5*(QQ+QL);
  end ELSE
  begin
    QL:=QQ;
    QQ:=0.5*(QQ+QH);
  END;
  GOTO 1;

5:WriteOutput;

  RR:=RR*XX*XX;
  writeln (OutputFile,' #');
  XL:=2*XL;
  WTd := DW-YN;
  WTe := DW-YJ;
  writeln (OutputFile,' The average hydraulic conductivity (Energy, m/d)'+
                      ' is: ',QQ:8:3);
  writeln (OutputFile,' #');
  writeln (OutputFile,' Midway hydr. head (energy, m) is: ',YJ:8:3);
  writeln (OutputFile,' Midway hydr. head (Darcy,  m) is: ',YN:8:3);
  writeln (OutputFile,' Midway W.T. depth (energy, m) is: ',WTe:8:3);
  writeln (OutputFile,' Midway W.T. depth (Darcy,  m) is: ',WTd:8:3);
 writeln (OutputFile,' #');
  closefile (OutputFile);

end; {Procedure EneCond}

end.


