unit UHydHead;

//Source code based on old Fortran file

interface
  uses SysUtils, Dialogs, UDMEnDrain, USpacing;
  var QL, QH, QQ : real;
      WTwarning : boolean;
  procedure CheckHeadData;
  procedure DefineExtra;
  procedure HydrHead;

implementation

uses UMainForm;

procedure CheckHeadData;
{----------------------}
var InputFile, ErrorFile : textfile;
begin
  assignfile(InputFile,DMEnDrain.FileName);
  reset (InputFile);
  readln (InputFile);
  readln (InputFile);
  readln (InputFile);
  readln (InputFile);
  readln (InputFile,RR,D1,D2,DW,DB,RE,WD,QA,QB1,QV1,QB2,QV2,XX);
  closefile (InputFile);
//Data checks
  assignfile(ErrorFile,'Error.lst');
  rewrite (ErrorFile);
  writeln (ErrorFile,' THE FOLLOWING ERRORS WERE FOUND IN THE INPUT:');
  writeln (ErrorFile);
  InputError := false;
  if (RR<=0) then
  begin
    writeln (ErrorFile,' Make sure that R > 0.');
    InputError := true;
  end;
  D1W := D1 - DW;
  if (D1W<0) then
  begin
    writeln (ErrorFile,' Make sure that D1 > Dw.');
    InputError := true;
  end;
  D21 := D2 - D1;
  if (D21<0) then
  begin
    writeln (ErrorFile,' Make sure that D2 > D1.');
    InputError := true;
  end;
  DBW := DB - DW;
  if (DBW<=0) then
  begin
    writeln (ErrorFile,' Make sure that Db > Dw.');
    InputError := true;
  end;
  DB1 := DB - D1;
  if (RE<0) then
  begin
    writeln (ErrorFile,' Make sure that E >= 0.');
    InputError := true;
  end;
  WX := WD/2;
(*
  if (D21<>0) and (D1<DB+WX) and (D1>DB) then
  begin
    writeln (ErrorFile,' Make sure that D1 > Db+Wd/2 or D1 < Db.');
    InputError := true;
  end;
  if (D2>DB) and (D2<DB+WX) then
  begin
    writeln (ErrorFile,' Make sure that D2 < Db or D2 > Db+Wd/2.');
    InputError := true;
  end;
*)
  if (QA<=RR) then
  begin
    writeln (ErrorFile,' Make sure that Ka >= R.');
    InputError := true;
  end;
  if (QV1<=RR) then
  begin
    QB1 := 0;
    QB2 := 0;
  end;
  if (QV2<=RR) then QB2 := 0;
  if InputError then
  begin
    writeln (ErrorFile);
    writeln (ErrorFile, 'Please correct the errors.');
  end;
  closefile (ErrorFile);
end; {procedure CheckHeadData}


Procedure DefineExtra;
{--------------------}
begin
  QQ1:=0;
  QQ2:=0;
  IF (QV1<=0) THEN QR1 := 1000000
  ELSE
  begin
    QR1:=sqrt(QB1/QV1);
    IF (QR1>0) then QQ1:=QB1/QR1;
  END;
  IF (QV2<=0) THEN QR2 := 1000000
  ELSE
  begin
    QR2:=sqrt(QB2/QV2);
    IF (QR2>0) then QQ2:=QB2/QR2;
  END;
  DR1:=D1W*QR1;
  DR2:=D21*QR2;
  DRT:=DR1+DR2;
  BWR1:=DBW*QR1;
//  B1R2:=DB1*QR2;
  B1R1:=DB1*QR1;
  IF (D1>DB) THEN
  begin
    DF:=WX-BWR1;
    IF (DF<0) then DF:=0;
    DG:=BWR1-WX;
    IF (DG<0) then DG:=0;
  end ELSE
  begin
    IF (D2>DB) THEN
    begin
//      DF:=WX-B1R2;
      DF:=WX-B1R1;
      IF (DF<0) then DF:=0;
//      DG:=B1R2-WX;
      DG:=B1R1-WX;
      IF (DG<0) then DG:=0;
    END else
    begin
      WX:=0;
      DF:=0;
//      DG:=B1R2;
      DG:=B1R1;
      IF (DG<0) then DG:=0
    END;
  END;
  PP:=0.00001;
  XX:=XX/2;
  NU:=round(XX/PP);
  KK:=1+round(WX/PP);
  IF (KK<2) then KK:=2;
  YR:=500;
  YH:=1000;
  YL:=0;
  YRP:=0;
  FE:=2*RR*RE*XX;
  RES:=0.0001;
  ZI:=0;
  YI:=0;
  WTwarning:=false;                         //required in MainForm, removable??
end; {procedure DefineExtra}


procedure HydrHead;
{-----------------}
label 1, 2, 3;
var IJ : integer;
    Passed0, Passed1, Passed2 : boolean;

begin
  InFileName:=DMEnDrain.FileName;
  OutFileName:=changefileext (DMEnDrain.FileName,'.out');
  assignfile(OutputFile,OutFileName);
  rewrite (OutputFile);
  writeln (OutputFile);
  writeln (OutputFile,' EnDrain, by R.J.Oosterbaan');
  writeln (OutputFile,' Program HydrHead calculates the shape of the water'+
                      ' table');
  writeln (OutputFile,' between drains using both the Darcy and the'+
                      ' energy balance');
  writeln (OutputFile,' equation');
  writeln (OutputFile,' The program allows the introduction of 3 soil layers'+
                      ' with');
  writeln (OutputFile,' anisotropic hydraulic conductivity.');
  writeln (OutputFile,' Calculations can be made for pipe drains or ditches');
  writeln (OutputFile,' with or without entrance resistance.');
  writeln (OutputFile);
  writeln (OutputFile,'  ',DMEnDrain.Title1);
  writeln (OutputFile,'  ',DMEnDrain.Title2);
  writeln (OutputFile);
  writeln (OutputFile,' Name of input file used:  ',InFileName);
  writeln (OutputFile,' Name of this output file: ',OutFileName);
  writeln (OutputFile);
  writeln (OutputFile,' HydrHead input data: ');
  writeln (OutputFile);
  writeln (OutputFile,' Time average recharge or discharge'+
                      '         R    (m/day) :  ',RR:10:4);
  writeln (OutputFile,' Bottom depth of 1st layer below s.s.'+
                      '       D1       (m) :  ',D1:10:2);
  writeln (OutputFile,' Bottom depth of 2nd layer below s.s.'+
                      '       D2       (m) :  ',D2:10:2);
  writeln (OutputFile,' Depth waterlevel in drain below s.s.'+
                      '       Dw       (m) :  ',DW:10:2);
  writeln (OutputFile,' Depth of drain bottom below s.s.'+
                      '           Db       (m) :  ',DB:10:2);
  writeln (OutputFile,' Entrance resistance at the drain'+
                      '           E    (day/m) :  ',RE:10:3);
  writeln (OutputFile,' Max. width of water body in the drain'+
                      '      W        (m) :  ',WD:10:3);
  writeln (OutputFile,' Hydraulic permeability, above drain level'+
                      '  Ka   (m/day) :  ',QA:10:3);
  writeln (OutputFile,' Horizontal permeability, 1st soil layer'+
                      '    Kb1  (m/day) :  ',QB1:10:3);
  writeln (OutputFile,' Vertical permeability, 1st soil layer'+
                      '      Kv1  (m/day) :  ',QV1:10:4);
  writeln (OutputFile,' Horizontal permeability, 2nd soil layer'+
                      '    Kb2  (m/day) :  ',QB2:10:3);
  writeln (OutputFile,' Vertical permeability, 2nd soil layer'+
                      '      Kv2  (m/day) :  ',QV2:10:4);
  writeln (OutputFile,' Spacing betweeen the drains'+
                      '                S        (m) :  ',XX:10:2);
  writeln (OutputFile);
  writeln (OutputFile,' Watertable height (see also the Symbols button):');
  writeln (OutputFile,'   X       F*        G*/p      F         G/p'+
                      '       T1        T2');
  writeln (OutputFile,' Output data:');

  DefineExtra;
  Passed0:=false;
  Passed1:=false;
  Passed2:=false;

//Calculating hydr. head Energy balance
1:for IJ:=KK to NU do
  begin
    XS:=PP*(IJ-0.5);                                  //XS =distance from drain
    YY:=1.571*XS;
    YZ:=YY+DG;
    FE:=2*RR*XX*RE;
    IF (IJ=KK) THEN
    BEGIN
      YI:=FE;
      YA:=FE;
    END ELSE
      YA:=YI+0.5*ZI*PP;                                  //YA is WT level at XS
    IF (D1>DB) THEN
    begin
      TR:=QA*YA+QV1*DF;                                  //TRA = transmissivity
      TRA:=TR+QQ1*YZ;
      IF (YZ>DR1) then TRA:=TR+QQ1*DR1+QQ2*(YZ-DR1);
      IF (YZ>DRT) then TRA:=TR+QQ1*DR1+QQ2*DR2;
    end ELSE
      IF (D2<=DB) THEN
         TRA:=QA*YA+QB1*D1W+QB2*D21
      ELSE
      begin
         TR:=QA*YA+QB1*D1W+QV2*DF;
         TRA:=TR+QQ2*YZ;
         IF (YZ>DR2) then TRA:=TR+QQ2*DR2;
      END;
    XC:=XX-XS;
    T1:=RR*XC/TRA;
    AY:=YA-YR;
    T2:=AY/XC;
    ZI:=T1+T2;
    YI:=YI+ZI*PP;
    IF (YI>YR) OR (ZI<0) then GOTO 2;
  end; { For IJ:=KK to NU do}

2:DIF:=ABS(YRP-YR);
  IF (DIF<RES) then GOTO 3;
  if not Passed0 and (XX<=25) and (Dif<1000*RES) then
  begin
    MainForm.WaitMemo.Lines.Add(' Still working ...');
    Passed0:=true;
  end;
  if not Passed1 then
     if ((DIF<100*RES) and (XX>25) and (XX<50)) or
        ((DIF<1000*RES) and (XX>=50)) then
        begin
          MainForm.WaitMemo.Lines.Add(' Still working ...');
          Passed1:=true;
        end;
  if not Passed2 and (DIF<10*RES) and (XX>=50) then
  begin
    MainForm.WaitMemo.Lines.Add(' Almost done .....');
    Passed2:=true;
  end;
  YRP:=YR;                      //final YI=YR is midway WT level energy balance
  IF (YI>YR) THEN
  begin
    YL:=YR;
    YR:=(YR+YH)/2;
  end ELSE
  begin
    YH:=YR;
    YR:=(YR+YL)/2;
  END;
  GOTO 1;

3:WriteOutput;
  IF (YI>DW) then
  begin
    showmessage (' Watertable above soil surface, adjust the data.');
    WTwarning:=true;
  end;
  writeln (OutputFile,' #');
  XL:=2*XL;
  WTd := DW-YN;
  WTe := DW-YI;
  writeln (OutputFile,' Midway hydr. head (energy, m) is: ',YI:8:3);
  writeln (OutputFile,' Midway hydr. head (Darcy,  m) is: ',YN:8:3);
  writeln (OutputFile,' Midway W.T. depth (energy, m) is: ',WTe:8:3);
  writeln (OutputFile,' Midway W.T. depth (Darcy,  m) is: ',WTd:8:3);
  writeln (OutputFile,' #');
  closefile (OutputFile);

end; {procedure HydrHead}

end.


